<?php

namespace Inventory\Adjustment\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Inventory\Adjustment\Http\Requests\ReasonRequest;
use Inventory\Adjustment\Interfaces\ReasonRepositoryInterface;
use Inventory\Adjustment\Models\Reason;

/**
 * Resource controller class for reason.
 */
class ReasonResourceController extends BaseController
{

    /**
     * Initialize reason resource controller.
     *
     * @param type ReasonRepositoryInterface $reason
     *
     * @return null
     */
    public function __construct(ReasonRepositoryInterface $reason)
    {
        parent::__construct();
        $this->repository = $reason;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Inventory\Adjustment\Repositories\Criteria\ReasonResourceCriteria::class);
    }

    /**
     * Display a list of reason.
     *
     * @return Response
     */
    public function index(ReasonRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Inventory\Adjustment\Repositories\Presenter\ReasonPresenter::class)
                ->$function();
        }

        $reasons = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('adjustment::reason.names'))
            ->view('adjustment::reason.index', true)
            ->data(compact('reasons', 'view'))
            ->output();
    }

    /**
     * Display reason.
     *
     * @param Request $request
     * @param Model   $reason
     *
     * @return Response
     */
    public function show(ReasonRequest $request, Reason $reason)
    {

        if ($reason->exists) {
            $view = 'adjustment::reason.show';
        } else {
            $view = 'adjustment::reason.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('adjustment::reason.name'))
            ->data(compact('reason'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new reason.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReasonRequest $request)
    {

        $reason = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('adjustment::reason.name')) 
            ->view('adjustment::reason.create', true) 
            ->data(compact('reason'))
            ->output();
    }

    /**
     * Create new reason.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReasonRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $reason                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('adjustment::reason.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('adjustment/reason/' . $reason->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/adjustment/reason'))
                ->redirect();
        }

    }

    /**
     * Show reason for editing.
     *
     * @param Request $request
     * @param Model   $reason
     *
     * @return Response
     */
    public function edit(ReasonRequest $request, Reason $reason)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('adjustment::reason.name'))
            ->view('adjustment::reason.edit', true)
            ->data(compact('reason'))
            ->output();
    }

    /**
     * Update the reason.
     *
     * @param Request $request
     * @param Model   $reason
     *
     * @return Response
     */
    public function update(ReasonRequest $request, Reason $reason)
    {
        try {
            $attributes = $request->all();

            $reason->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('adjustment::reason.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('adjustment/reason/' . $reason->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('adjustment/reason/' . $reason->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the reason.
     *
     * @param Model   $reason
     *
     * @return Response
     */
    public function destroy(ReasonRequest $request, Reason $reason)
    {
        try {

            $reason->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('adjustment::reason.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('adjustment/reason/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('adjustment/reason/' . $reason->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple reason.
     *
     * @param Model   $reason
     *
     * @return Response
     */
    public function delete(ReasonRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('adjustment::reason.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('adjustment/reason'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/adjustment/reason'))
                ->redirect();
        }

    }

    /**
     * Restore deleted reasons.
     *
     * @param Model   $reason
     *
     * @return Response
     */
    public function restore(ReasonRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('adjustment::reason.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/adjustment/reason'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/adjustment/reason/'))
                ->redirect();
        }

    }

}
